package com.poc.kafka.mongo.service;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.support.SendResult;
import org.springframework.stereotype.Service;
import org.springframework.util.concurrent.ListenableFuture;
import org.springframework.util.concurrent.ListenableFutureCallback;

import com.google.gson.Gson;
import com.poc.kafka.mongo.entity.Employee;
import com.poc.kafka.mongo.mapper.EmployeeMapper;
import com.poc.kafka.mongo.model.Emp;
import com.poc.kafka.mongo.repository.EmployeeMongoRepository;
import com.poc.kafka.mongo.repository.EmployeeRepository;
@Service
public class EmployeeServiceImpl implements EmployeeService {
	private static final Logger logger = LoggerFactory.getLogger(EmployeeServiceImpl.class);
	@Autowired
	private KafkaTemplate<String, String> kafkaTemplate;

	@Autowired
	private Gson gson;
	@Value(value = "${spring.kafka.topics}")
	private String topicName;

	@Autowired
	private EmployeeMongoRepository employeeMongoRepository;
	@Autowired
	private EmployeeMapper employeeMapper;
	@Autowired
	private EmployeeRepository employeeRepository;


	@Override
	public void send(Emp emp) {

		String request = gson.toJson(emp);
		ListenableFuture<SendResult<String, String>> future = this.kafkaTemplate.send(topicName, request);
		future.addCallback(new ListenableFutureCallback<SendResult<String, String>>() {
			@Override
			public void onFailure(Throwable ex) {
				logger.info("Unable to send message=[ {} ] due to : {}", request, ex.getMessage());
			}

			@Override
			public void onSuccess(SendResult<String, String> result) {
				logger.info("Sent message=[ {} ] with offset=[ {} ]", request, result.getRecordMetadata().offset());
			}
		});
	}

	@Override
	@KafkaListener(topics = { "${spring.kafka.topics}" })
	public void receive(String emp) {
		logger.info("Kafka event consumed is:{} " , emp);
		Emp e = gson.fromJson(emp, Emp.class);
		logger.info("converted value:{} " , e.toString());
		/**
		 * To send to mongo database after listening the record from kafka
		 */
		saveOrUpdate(e);
	}
	
	@Override
	public Emp findByEmpId(Long empId) {
		return (Emp)employeeMapper.mapEntityToPojo(employeeMongoRepository.findByEmpId(empId));
	}

	@Override
	public Emp findById(String id) {
		return (Emp)employeeMapper.mapEntityToPojo(employeeMongoRepository.findById(id).orElse(new Employee()));
	}

	@Override
	public List<Emp> findAll() {
		List<Employee> list= employeeMongoRepository.findAll();
		List<Emp> empList= list.stream()
				.map(e->employeeMapper.mapEntityToPojo(e))
				.map(e->(Emp)e)
				.collect(Collectors.toList());
		return empList;
	}

	@Override
	public Emp save(Emp emp) {
		Employee employee = (Employee) employeeMapper.mapPojoToEntity(emp);
		return (Emp)employeeMapper.mapEntityToPojo(employeeMongoRepository.save(employee));
	}

	@Override
	public void saveOrUpdate(Emp emp) {
		Employee employee = (Employee) employeeMapper.mapPojoToEntity(emp);
		employeeRepository.saveOrUpdate(employee);

	}

	@Override
	public List<Emp> findBySalary(Double sal) {
		List<Emp> empList= null;
		try (Stream<Employee> stream = employeeMongoRepository.findBySalary(sal)) {
			List<Employee> list = stream.collect(Collectors.toList());
			empList= list.stream()
					.map(e->(Emp)employeeMapper.mapEntityToPojo(e))
					.collect(Collectors.toList());
		}
		return empList;
	}

	@Override
	public Long deleteByEmpId(Long empId) {
		return employeeMongoRepository.deleteByEmployeeId(empId);
	}



}
